from arepo.models import Base

from sqlalchemy.orm import relationship
from sqlalchemy import Integer, Column, String, Boolean, ForeignKey, PrimaryKeyConstraint


class ConfigurationModel(Base):
    __tablename__ = "configuration"

    id = Column('id', String, primary_key=True)
    vulnerable = Column('vulnerable', Boolean, nullable=True)
    part = Column('part', String, nullable=False)
    version = Column('version', String, nullable=True)
    update = Column('update', String, nullable=True)
    edition = Column('edition', String, nullable=True)
    language = Column('language', String, nullable=True)
    sw_edition = Column('sw_edition', String, nullable=True)
    target_sw = Column('target_sw', String, nullable=True)
    target_hw = Column('target_hw', String, nullable=True)
    other = Column('other', String, nullable=True)
    vulnerability_id = Column(String, ForeignKey('vulnerability.id'))
    vendor_id = Column(String, ForeignKey('vendor.id'))
    product_id = Column(String, ForeignKey('product.id'))


class VendorModel(Base):
    __tablename__ = "vendor"

    id = Column('id', String, primary_key=True)
    name = Column('name', String, nullable=False)
    products = relationship("ProductModel", backref="vendor")


class ProductModel(Base):
    __tablename__ = "product"

    id = Column('id', String, primary_key=True)
    name = Column('name', String, nullable=False)
    vendor_id = Column(String, ForeignKey('vendor.id'))
    product_type_id = Column(Integer, ForeignKey('product_type.id'))
    configurations = relationship("ConfigurationModel", backref="product")


class ProductTypeModel(Base):
    __tablename__ = "product_type"

    id = Column('id', Integer, primary_key=True)
    name = Column('name', String, nullable=False)
    products = relationship("ProductModel", backref="product_type")


class ConfigurationVulnerabilityModel(Base):
    __tablename__ = 'configuration_vulnerability'
    __table_args__ = (
        PrimaryKeyConstraint('configuration_id', 'vulnerability_id'),
    )

    configuration_id = Column('configuration_id', String, ForeignKey('configuration.id'))
    vulnerability_id = Column('vulnerability_id', String, ForeignKey('vulnerability.id'))



from arepo.models import Base

from sqlalchemy.orm import relationship
from sqlalchemy import Column, Integer, String, ForeignKey, Float, DateTime, ForeignKeyConstraint


class TagModel(Base):
    __tablename__ = "tag"

    id = Column('id', Integer, primary_key=True)
    name = Column('name', String, nullable=False)
    references = relationship("ReferenceModel", secondary="reference_tag", backref='tags')


class ReferenceModel(Base):
    __tablename__ = "reference"

    id = Column('id', String, primary_key=True)
    url = Column('url', String, nullable=False)
    vulnerability_id = Column(String, ForeignKey('vulnerability.id'))


class ReferenceTagModel(Base):
    __tablename__ = 'reference_tag'
    __table_args__ = (
        ForeignKeyConstraint(('reference_id',), ['reference.id']),
        ForeignKeyConstraint(('tag_id',), ['tag.id']),
    )

    reference_id = Column(String, ForeignKey('reference.id'), primary_key=True)
    tag_id = Column(Integer, ForeignKey('tag.id'), primary_key=True)


class VulnerabilityModel(Base):
    __tablename__ = "vulnerability"

    id = Column('id', String, primary_key=True)
    description = Column('description', String, nullable=True)
    assigner = Column('assigner', String, nullable=False)
    severity = Column('severity', String, nullable=True)
    exploitability = Column('exploitability', Float, nullable=True)
    impact = Column('impact', Float, nullable=True)
    published_date = Column('published_date', DateTime, nullable=False)
    last_modified_date = Column('last_modified_date', DateTime, nullable=False)
    references = relationship("ReferenceModel", backref="vulnerability")
    configurations = relationship("ConfigurationModel", backref="vulnerability")
    commits = relationship("CommitModel", backref="vulnerability")


class VulnerabilityCWEModel(Base):
    __tablename__ = 'vulnerability_cwe'
    __table_args__ = (
        ForeignKeyConstraint(('vulnerability_id',), ['vulnerability.id']),
        ForeignKeyConstraint(('cwe_id',), ['cwe.id'])
    )

    vulnerability_id = Column(String, ForeignKey('vulnerability.id'), primary_key=True)
    cwe_id = Column(Integer, ForeignKey('cwe.id'), primary_key=True)
